import json

from .exception import Tracking51Exception

def process_response(response):
    try:
        json_content = response.json()
    except json.JSONDecodeError:
        raise Tracking51Exception('response json type conversion failed')
    return json_content


import tracking51

from .exception import Tracking51Exception

from .const import ErrEmptyAPIKey

def get_api_key():
    '''Get 51Tracking API key'''
    if tracking51.api_key is not None and tracking51.api_key != '':
        return tracking51.api_key
    else:
        raise Tracking51Exception(ErrEmptyAPIKey)

def build_query(params):
    kv_pairs = [f'{key}={value}' for key, value in params.items()]

    result = '&'.join(kv_pairs)

    return result

def is_empty(value):
    if value is None:
        return True
    if isinstance(value, (str, list, tuple, dict)):
        return not bool(value)
    return False



ErrEmptyAPIKey = 'API Key is missing';
ErrMissingTrackingNumber      = 'Tracking number cannot be empty';
ErrMissingCourierCode         = 'Courier Code cannot be empty';
ErrMissingAwbNumber           = 'Awb number cannot be empty';
ErrMaxTrackingNumbersExceeded = 'Max. 40 tracking numbers create in one call';
ErrEmptyId                    = 'Id cannot be empty';
ErrInvalidAirWaybillFormat    = 'The air waybill number format is invalid and can only be 12 digits in length';




from .const import ErrMissingTrackingNumber

from .request import make_request

from .exception import Tracking51Exception

from .util import is_empty

__all__ = ['get_all_couriers', 'detect']

api_modul = 'couriers'

def get_all_couriers():
    path = api_modul + '/all'
    response = make_request('GET', path, None)
    return response

def detect(params):
    if is_empty(params.get('tracking_number')):
        raise Tracking51Exception(ErrMissingTrackingNumber)
    path = api_modul + '/detect'
    response = make_request('POST', path, params)
    return response



class Tracking51Exception(Exception):
    def __init__(self, message):
        super(Tracking51Exception, self).__init__()
        self.message = message

    def __str__(self):
        return '{}: {}'.format(self.__class__.__name__, self.message)


from .const import ErrMissingTrackingNumber, ErrMissingCourierCode, ErrMaxTrackingNumbersExceeded, ErrEmptyId

from .exception import Tracking51Exception

from .request import make_request

from .util import is_empty

__all__ = ['create_tracking', 'get_tracking_results', 'batch_create_trackings', 'update_tracking_by_id', 'delete_tracking_by_id', 'retrack_tracking_by_id']

api_modul = 'trackings'

def create_tracking(params):
    if is_empty(params.get('tracking_number')):
        raise Tracking51Exception(ErrMissingTrackingNumber)
    if is_empty(params.get('courier_code')):
        raise Tracking51Exception(ErrMissingCourierCode)
    path = api_modul + '/create'
    response = make_request('POST', path, params)
    return response

def get_tracking_results(params):
    path = api_modul + '/get'
    response = make_request('GET', path, params)
    return response

def batch_create_trackings(params):
    if len(params) > 40:
        raise Tracking51Exception(ErrMaxTrackingNumbersExceeded)

    for item in params:
        if is_empty(item.get('tracking_number')):
            raise Tracking51Exception(ErrMissingTrackingNumber)
        if is_empty(item.get('courier_code')):
            raise Tracking51Exception(ErrMissingCourierCode)

    path = api_modul + '/batch'
    response = make_request('POST', path, params)
    return response

def update_tracking_by_id(id_string,params):
    if is_empty(id_string):
        raise Tracking51Exception(ErrEmptyId)
    path = api_modul + '/update/' + id_string
    response = make_request('PUT', path, params)
    return response

def delete_tracking_by_id(id_string):
    if is_empty(id_string):
        raise Tracking51Exception(ErrEmptyId)
    path = api_modul + '/delete/' + id_string
    response = make_request('DELETE', path, None)
    return response

def retrack_tracking_by_id(id_string):
    if is_empty(id_string):
        raise Tracking51Exception(ErrEmptyId)
    path = api_modul + '/retrack/' + id_string
    response = make_request('POST', path, None)
    return response


import json

import requests

from .util import get_api_key,build_query

from .exception import Tracking51Exception

from .response import process_response

apiBaseUrl = 'api.51Tracking.com'

apiPort = 443

apiVersion = 'v4'

timeout = 10

headers = {}

def get_request_url(path):
    pact = 'https' if apiPort == 443 else 'http'
    url = pact+'://'+apiBaseUrl+'/'+apiVersion+'/'+path
    return url

def get_request_header(apiKey):
    headers['Accept'] = 'application/json'
    headers['Content-Type'] = 'application/json'
    headers['Tracking-Api-Key'] = apiKey
    return headers

def make_request(method='GET', path='', params=None):
    try:
        url = get_request_url(path)

        headers = get_request_header(get_api_key())

        json_data = None
        if params is not None:
            if method == 'GET':
                url = url + '?' +build_query(params)
            else:
                json_data = json.dumps(params)

        if method == 'GET':
            response = requests.get(url, headers=headers, timeout=timeout)
        elif method == 'POST':
            response = requests.post(url, headers=headers, data=json_data, timeout=timeout)
        elif method == 'PUT':
            response = requests.put(url, headers=headers, data=json_data, timeout=timeout)
        elif method == 'DELETE':
            response = requests.delete(url, headers=headers, timeout=timeout)
        else:
            raise Tracking51Exception('Unsupported HTTP method: {}'.format(method))

        return process_response(response)

    except requests.exceptions.RequestException as e:
        print('Error making request:', e)
        return None


from .const import ErrInvalidAirWaybillFormat, ErrMissingAwbNumber

from .exception import Tracking51Exception

from .request import make_request

from .util import is_empty

__all__ = ['create_an_air_waybill']

def create_an_air_waybill(params):
    if is_empty(params.get('awb_number')):
        raise Tracking51Exception(ErrMissingAwbNumber)
    if len(params['awb_number']) != 12:
        raise Tracking51Exception(ErrInvalidAirWaybillFormat)
    path = '/awb'
    response = make_request('POST', path, params)
    return response